from paralelocs_qlikapi.method.request import QlikRequest
import requests

class Base():
    """
    Classe base para os métodos da API


    """

    def __init__(self,
                 auth= None,
                 session=None):

        self.headers = auth.headers
        self.uri = auth.uri
        self.sslfile = auth.sslfile
        self.sslfile_key = auth.sslfile_key
        self.xrfkey = auth.xrfkey
        self.verify = auth.verify
        self.session = session
        self.auth = auth

    def get(self):
        raise NotImplementedError

    def search(self):
        raise NotImplementedError

    def post(self):
        raise NotImplementedError

class DefaultMethods(Base):

    def send(self, method = None, resource= None, headers=None, data = None, params= None):
        
        if(params is None):
            url = f'{self.uri}{resource}?xrfkey={self.xrfkey}'
        else:
            url = f"{self.uri}{resource}?xrfkey={self.xrfkey}&{params}"

        if(headers is None):
            headers = self.headers
        else:
            customHeaders = self.headers.copy()
            customHeaders.update(headers)
            headers = customHeaders.copy()

        request = requests.Request(method=method, url=url, data=data, headers=headers)
        prepared = request.prepare()
        response = self.session.send(
                                            prepared, 
                                            cert=(self.sslfile, self.sslfile_key), 
                                            verify=self.verify
                                            )

        try:
            self.response = dict(
                    status_code = response.status_code,
                    content = response.json()
            )
        except:
            self.response = dict(
                    status_code = response.status_code,
                    content = response.content.decode('utf-8')
            ) 
            
        finally:
            return self.response

        

    def get(self):
        self.method = 'GET'
        return self.send(method = self.method, resource= self.resource)

    def search(self, name):
        self.method = 'GET'
        params = "filter=name eq '{name}'"
        return self.send(method = self.method, resource=self.resource, params=params)


    def search_by_id(self, id):
        self.method = 'GET'
        resource = f'{self.resource}/{id}'
        return self.send(method = self.method, resource= resource)

    def delete(self, id):
        method = 'DELETE'
        resource = f'{self.resource}/{id}'
        return self.send(method = method, resource= resource)


from paralelocs_qlikapi.method.base import DefaultMethods, QlikRequest

class Task(DefaultMethods):
    """
    Classe para trabalhar com o recurso APP da API do QLIK

    """

    def __init__(self,
                 auth= None,
                 session=None):
        super().__init__(auth=auth, session=session)
        self.resource = 'task'


    def start_by_name(self, name):
        method = 'POST'
        resource = 'task/start'
        params = f'name={name}'
        return self.send(method = method, resource= self.resource, params=params)


    def start_by_id(self, id):
        method = 'POST'
        resource = f'task/{id}/start'
        return self.send(method = method, resource= self.resource)



class QlikAuth():
    """

    Classe base para criação dos Headers exigidos pela API do QLIK

    """

    def __init__(self,
                 hostname='localhost',
                 sslfile=None,
                 sslfile_key=None,
                 verify=True
                 ):

        self.xrfkey = '0123456789abcdef'
        self.hostname = hostname
        self.uri = f'https://{hostname}:4242/qrs/'
        self.verify = verify
        self.headers = {}
        self.headers['x-qlik-xrfkey']= self.xrfkey
        self.headers['X-Qlik-User'] = 'UserDirectory=internal;UserId=sa_repository'
        self.headers['Content-Type']= 'application/json'
        self.headers['Cache-Control']= 'no-cache'
        self.headers['Accept']='*/*'
        self.sslfile = sslfile
        self.sslfile_key = sslfile_key


from paralelocs_qlikapi.api.auth import QlikAuth
from paralelocs_qlikapi.method import app, reloadtask, task, tag, stream
import requests



class QlikAPI():
    """

        Classe de High Level com a abstração de toda a complexidade da API.
        Todos os metodos serão disponibilizados nessa classe

    """

    def __init__(self,
                 hostname='localhost',
                 sslfile=None,
                 sslfile_key=None,
                 verify=True
                 ):

        self.auth = QlikAuth(
                                        hostname=hostname,
                                        sslfile=sslfile,
                                        sslfile_key=sslfile_key,
                                        verify=verify
                                            )
        self.session = requests.Session()
        self.app = app.App(auth = self.auth, session=self.session)
        self.reloadtask = reloadtask.Task(auth = self.auth, session=self.session)
        self.task = task.Task(auth = self.auth, session=self.session)
        self.tag = tag.Tag(auth = self.auth, session=self.session)
        self.stream = stream.Stream(auth = self.auth, session=self.session)



from paralelocs_qlikapi.method.base import DefaultMethods, QlikRequest
import json

class Task(DefaultMethods):
    """
    Classe para trabalhar com o recurso APP da API do QLIK

    """

    def __init__(self,
                 auth= None,
                 session=None):
        super().__init__(auth=auth, session=session)
        self.resource = 'reloadtask'

    def post(self):
        method = 'POST'
        resource = 'reloadtask/create'
        body = self.create_body_dict()
        data = json.dumps(body)
        return self.send(method = method, resource= resource, data=data)
       

        

    def create_composite_events_dict(self):
        compositeEvents = []
        compositeRules = []

        compositeRules.append(
                                dict(
                                        reloadTask= dict(id='6546fad6-e0e1-4bcd-8eaa-995e9cafe0c7'),
                                        ruleState = 1
                                    )
                            )
        union = dict(
                timeConstraint = dict(days=0, hours=0,minutes=360,seconds=0),
                compositeRules = compositeRules,
                name = "Event Trigger",
                enabled = True,
                eventType = 1
            )
        
        compositeEvents.append(union)

        return compositeEvents


    def create_task_dict(self):
        body = dict(
            
                    app= dict(
                                id= "1e048110-99a6-43d7-bac3-391070202967",
                                ), 
                    name="Reload Goto Analytics",
                    taskSessionTimeout= 1440,
                    maxRetries = 0,
                    enable= True
                    )
        return body

    def create_body_dict(self):

        body = dict(
            task = self.create_task_dict(),
            compositeEvents = self.create_composite_events_dict(),
            schemaEvents = self.create_schema_events_dict()
        )

        return body

    def create_schema_events_dict(self):
        schemaEvents = []
        schemaFilterDescription = ["* * - * * * * *"]

        body = dict(
            timeZone = "America/New_York",
            daylightSavingTime = 0,
            startDate = "2017-01-11T12:05:46.000",
            expirationDate = "9999-01-01T00:00:00.000",
            schemaFilterDescription = schemaFilterDescription,
            incrementDescription = "0 0 1 0",
            incrementOption = 2,
            name = "Daily",
            enabled = True
        )
        
        schemaEvents.append(body)

        return schemaEvents



    def reload(self, name):
        app = self.search(name=name)
        app = app['content'][0]
        appid = app['id']
        resource = f'app/{appid}/reload'
        method = 'POST'
        data = ''
        
        return self.send(method = method, resource= resource, data=data)


import requests
import json
from paralelocs_qlikapi.method.base import DefaultMethods, QlikRequest
from paralelocs_qlikapi.method.tag import Tag
from paralelocs_qlikapi.method.stream import Stream

class App(DefaultMethods):
    """
    Classe para trabalhar com o recurso APP da API do QLIK

    """

    def __init__(self,
                 auth= None,
                 session=None):
        super().__init__(auth=auth, session=session)
        self.resource = 'app'

    def post(self, path, name, filename):

        resource = 'app/upload'
        method = 'POST'
        headers = {}
        headers['Content-Type']= 'application/vnd.qlik.sense.app'
        queryparams = f'name={name}&keepdata=false'

        try:
            data = open(f'{path}/{filename}','rb').read()
        except FileNotFoundError as e:
            print(f'{e}\n QVF file not found.')
            data = ''
        
        return self.send(method = method, resource=resource, params=queryparams, headers=headers, data=data)


    def reload(self, name):
        app = self.search(name=name)
        app = app['content'][0]
        appid = app['id']
        resource = f'app/{appid}/reload'
        method = 'POST'
        data = ''
        self.url = f'{self.uri}{self.resource}?xrfkey={self.xrfkey}'
        self.request = QlikRequest(self)
        return self.send(method = method, resource=resource, data=data)

    # INCLUIR TAG BY APP ID
    def add_tag(self, appid, tagname):

        ##### RECUPERA DADOS DA APP ###########

        # appname = self.search(name=appname)
        # exist_app = appname['content'][0]
        # appid = exist_app['id']
        
        app = self.search_by_id(id=appid)['content']

        ##### VERIFICA SE TAGS EXISTE ###########
        tagapi = Tag(auth = self.auth, session= self.session)
        existing_tags = tagapi.search(name = tagname)['content']

        if (len(existing_tags) > 0):
            _tag = existing_tags[0]
        else:
            _tag = tagapi.post(name=tagname)['content']
            

        ##### DEFINI RECURSO E METODO ###########    
        resource = f'app/{appid}'
        method = 'PUT'
        

        ###### CRIA BODY COM TAG ################
        tags = []
        tags.append(_tag)
        tags_dict = dict(tags = tags)
        app.update(tags_dict)
        data = json.dumps(app)

        return self.send(method = method, resource=resource, data=data)

    # INCLUIR PUBLISH BY APP ID
    def publish_to_stream(self, appid, streamname):
        """

            PUBLICAR APP NA STREAM


        """

        ##### RECUPERA DADOS DA APP ###########

        # appname = self.search(name=appname)
        # exist_app = appname['content'][0]
        # appid = exist_app['id']

        ##### VERIFICA SE EXISTE STREAM ###########
        streamapi = Stream(auth = self.auth, session= self.session)
        existing_stream = streamapi.search(name = streamname)['content']

        if (len(existing_stream) > 0):
            _stream= existing_stream[0]['id']
        else:
            raise AttributeError ('Stream not found')
                   

        ##### DEFINI RECURSO E METODO ###########    
        resource = f'app/{appid}/publish'
        method = 'PUT'

        ###### CRIA QUERY PARAMTERS ################
        params = f'stream={_stream}'

        return self.send(method = method, resource=resource, params=params)


    def unpublish(self, name):

        """
            DESPUBLICAR APP

        """
        
        app = self.search(name=name)
        app = app['content'][0]
        appid = app['id']
        resource = f'app/{appid}/unpublish'
        method = 'POST'
        data = ''
        return self.send(method = method, resource=resource, data=data)



from paralelocs_qlikapi.method.base import DefaultMethods, QlikRequest
import json


class Stream(DefaultMethods):
    """
    Classe para trabalhar com o recurso TAG da API do QLIK

    """

    def __init__(self,
                 auth= None,
                 session=None):
        super().__init__(auth=auth, session=session)
        self.resource = 'stream'

    def post(self, name):
        method = 'POST'
        body = dict(
                name = name,
        )
        data = json.dumps(body)
        
        return self.send(method = method, resource= self.resource, data=data)


"""Top-level package for Python Qlik API QRS."""

from __future__ import absolute_import
from paralelocs_qlikapi import api
from paralelocs_qlikapi import method

__author__ = """Paralelo Consultoria e Serviços Ltda"""
__email__ = 'contato@paralelocs.com.br'
__version__ = '0.2.0'
